import { G as GenerateDocumentationOptions, a as GeneratedDoc, D as DocEntry, B as BaseTypeTableProps } from './type-table-DyymKQK8.js';
export { b as GenerateOptions, d as generate, c as generateDocumentation, g as getProject } from './type-table-DyymKQK8.js';
import fg from 'fast-glob';
import { Nodes } from 'hast';
import { Root } from 'mdast';
import { Transformer } from 'unified';
import 'ts-morph';

interface Templates {
    block: (doc: GeneratedDoc, children: string) => string;
    property: (entry: DocEntry) => string;
}
interface GenerateMDXOptions extends GenerateDocumentationOptions {
    /**
     * a root directory to resolve relative file paths
     */
    basePath?: string;
    templates?: Partial<Templates>;
}
declare function generateMDX(source: string, { basePath, templates: overrides, ...rest }?: GenerateMDXOptions): string;

interface GenerateFilesOptions {
    input: string | string[];
    /**
     * Output directory, or a function that returns the output path
     */
    output: string | ((inputPath: string) => string);
    globOptions?: fg.Options;
    options?: GenerateMDXOptions;
    /**
     * @returns New content
     */
    transformOutput?: (path: string, content: string) => string;
}
declare function generateFiles(options: GenerateFilesOptions): Promise<void>;

declare function renderMarkdownToHast(md: string): Promise<Nodes>;

interface RemarkAutoTypeTableOptions {
    /**
     * @defaultValue 'auto-type-table'
     */
    name?: string;
    /**
     * @defaultValue 'TypeTable'
     */
    outputName?: string;
    renderMarkdown?: typeof renderMarkdownToHast;
    /**
     * Override some type table props
     */
    options?: BaseTypeTableProps['options'];
}
/**
 * Compile `auto-type-table` into Fumadocs UI compatible TypeTable
 *
 * MDX is required to use this plugin.
 */
declare function remarkAutoTypeTable({ name, outputName, renderMarkdown, options, }?: RemarkAutoTypeTableOptions): Transformer<Root, Root>;

export { DocEntry, GenerateDocumentationOptions, type GenerateFilesOptions, type GenerateMDXOptions, GeneratedDoc, type RemarkAutoTypeTableOptions, generateFiles, generateMDX, remarkAutoTypeTable, renderMarkdownToHast };
