import {
  __async,
  __objRest,
  __spreadProps,
  __spreadValues,
  generate,
  generateDocumentation,
  getProject,
  getTypeTableOutput,
  renderMarkdownToHast
} from "./chunk-LRCT2FYJ.js";

// src/lib/mdx.ts
import * as path from "node:path";
import fs from "node:fs";
var regex = new RegExp("^---type-table---\\r?\\n(?<file>.+?)(?:#(?<name>.+))?\\r?\\n---end---$", "gm");
var defaultTemplates = {
  block: (doc, c) => `### ${doc.name}

${doc.description}

<div className='*:border-b [&>*:last-child]:border-b-0'>${c}</div>`,
  property: (c) => `<div className='text-sm text-fd-muted-foreground py-4'>

<div className="flex flex-row items-center gap-4">
  <code className="text-sm">${c.name}</code>
  <code className="text-fd-muted-foreground">{${JSON.stringify(c.type)}}</code>
</div>

${c.description || "No Description"}

${Object.entries(c.tags).map(([tag, value]) => `- ${tag}:
${replaceJsDocLinks(value)}`).join("\n")}

</div>`
};
function generateMDX(source, _a = {}) {
  var _b = _a, { basePath = "./", templates: overrides } = _b, rest = __objRest(_b, ["basePath", "templates"]);
  var _a2;
  const templates = __spreadValues(__spreadValues({}, defaultTemplates), overrides);
  const project = (_a2 = rest.project) != null ? _a2 : getProject(rest.config);
  return source.replace(regex, (...args) => {
    const groups = args[args.length - 1];
    const file = path.resolve(basePath, groups.file);
    const content = fs.readFileSync(file);
    const docs = generateDocumentation(file, groups.name, content.toString(), __spreadProps(__spreadValues({}, rest), {
      project
    }));
    return docs.map(
      (doc) => templates.block(doc, doc.entries.map(templates.property).join("\n"))
    ).join("\n\n");
  });
}
function replaceJsDocLinks(md) {
  return md.replace(new RegExp("{@link (?<link>[^}]*)}", "g"), "$1");
}

// src/lib/file.ts
import * as path2 from "node:path";
import { mkdir, writeFile, readFile } from "node:fs/promises";
import fg from "fast-glob";
function generateFiles(options) {
  return __async(this, null, function* () {
    var _a, _b, _c;
    const files = yield fg(options.input, options.globOptions);
    const project = (_c = (_a = options.options) == null ? void 0 : _a.project) != null ? _c : getProject((_b = options.options) == null ? void 0 : _b.config);
    const produce = files.map((file) => __async(this, null, function* () {
      const absolutePath = path2.resolve(file);
      const outputPath = typeof options.output === "function" ? options.output(file) : path2.resolve(
        options.output,
        `${path2.basename(file, path2.extname(file))}.mdx`
      );
      const content = (yield readFile(absolutePath)).toString();
      let result = generateMDX(content, __spreadProps(__spreadValues({
        basePath: path2.dirname(absolutePath)
      }, options.options), {
        project
      }));
      if (options.transformOutput) {
        result = options.transformOutput(outputPath, result);
      }
      yield write(outputPath, result);
      console.log(`Generated: ${outputPath}`);
    }));
    yield Promise.all(produce);
  });
}
function write(file, content) {
  return __async(this, null, function* () {
    yield mkdir(path2.dirname(file), { recursive: true });
    yield writeFile(file, content);
  });
}

// src/lib/remark-auto-type-table.tsx
import { valueToEstree } from "estree-util-value-to-estree";
import { visit } from "unist-util-visit";
import { toEstree } from "hast-util-to-estree";
import { dirname as dirname2 } from "node:path";
function expressionToAttribute(key, value) {
  return {
    type: "mdxJsxAttribute",
    name: key,
    value: {
      type: "mdxJsxAttributeValueExpression",
      data: {
        estree: {
          type: "Program",
          body: [
            {
              type: "ExpressionStatement",
              expression: value
            }
          ]
        }
      }
    }
  };
}
function mapProperty(entry, renderMarkdown) {
  return __async(this, null, function* () {
    const value = valueToEstree({
      type: entry.type,
      default: entry.tags.default || entry.tags.defaultValue
    });
    if (entry.description) {
      const hast = toEstree(yield renderMarkdown(entry.description), {
        elementAttributeNameCase: "react"
      }).body[0];
      value.properties.push({
        type: "Property",
        method: false,
        shorthand: false,
        computed: false,
        key: {
          type: "Identifier",
          name: "description"
        },
        kind: "init",
        value: hast.expression
      });
    }
    return {
      type: "Property",
      method: false,
      shorthand: false,
      computed: false,
      key: {
        type: "Identifier",
        name: entry.name
      },
      kind: "init",
      value
    };
  });
}
function remarkAutoTypeTable({
  name = "auto-type-table",
  outputName = "TypeTable",
  renderMarkdown = renderMarkdownToHast,
  options = {}
} = {}) {
  var _a;
  const project = (_a = options.project) != null ? _a : getProject(options.config);
  return (tree, file) => __async(this, null, function* () {
    const queue = [];
    let basePath = options == null ? void 0 : options.basePath;
    if (!basePath && file.path) basePath = dirname2(file.path);
    visit(tree, "mdxJsxFlowElement", (node) => {
      if (node.name !== name) return;
      const props = {};
      for (const attr of node.attributes) {
        if (attr.type !== "mdxJsxAttribute" || typeof attr.value !== "string")
          throw new Error(
            "`auto-type-table` does not support non-string attributes"
          );
        props[attr.name] = attr.value;
      }
      function run() {
        return __async(this, null, function* () {
          const output = yield getTypeTableOutput(__spreadProps(__spreadValues({}, props), {
            options: __spreadProps(__spreadValues({}, options), {
              project,
              basePath
            })
          }));
          const rendered = output.map((doc) => __async(this, null, function* () {
            const properties = yield Promise.all(
              doc.entries.map((entry) => mapProperty(entry, renderMarkdown))
            );
            return {
              type: "mdxJsxFlowElement",
              name: outputName,
              attributes: [
                expressionToAttribute("type", {
                  type: "ObjectExpression",
                  properties
                })
              ],
              data: {
                // for Fumadocs `remarkStructure`
                _string: [
                  doc.name,
                  doc.description,
                  ...doc.entries.flatMap((entry) => [
                    `${entry.name}: ${entry.type}`,
                    entry.description
                  ])
                ]
              },
              children: []
            };
          }));
          Object.assign(node, {
            type: "root",
            attributes: [],
            children: yield Promise.all(rendered)
          });
        });
      }
      queue.push(run());
      return "skip";
    });
    yield Promise.all(queue);
  });
}
export {
  generate,
  generateDocumentation,
  generateFiles,
  generateMDX,
  getProject,
  remarkAutoTypeTable,
  renderMarkdownToHast
};
